/* Programmname                                                             */

/*--------------------------------------------------------------------------*/
/* System Includes                                                          */
/*--------------------------------------------------------------------------*/

	#include <clib/exec_protos.h>
	#include <clib/alib_protos.h>
	#include <clib/intuition_protos.h>
	#include <clib/graphics_protos.h>
	#include <clib/dos_protos.h>
	#include <exec/memory.h>
	#include <cybergraphx/cybergraphics.h>

	#include <stdlib.h>
	#include <string.h>
	#include <stdio.h>

/*--------------------------------------------------------------------------*/
/* Konstanten und Makros                                                    */
/*--------------------------------------------------------------------------*/

	#define WINX 80
	#define WINY 40
	#define WINW 200
	#define WINH 100

	#define BOBW 80
	#define BOBH 40
	#define BOBMINX 40
	#define BOBMINY 20
	#define BOBMAXX (BOBMINX+BOBW-1)
	#define BOBMAXY (BOBMINY+BOBH-1)

/*--------------------------------------------------------------------------*/
/* Typdefinitionen                                                          */
/*--------------------------------------------------------------------------*/

	struct bob {
		struct RastPort *rp;
		short x;
		short y;
		short w;
		short h;
		struct BitMap *bm;
		short backx;
		short backy;
		short backw;
		short backh;
		struct BitMap *back;
		};

/*--------------------------------------------------------------------------*/
/* Globale Variablen                                                        */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/* Prototypen                                                               */
/*--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*--------------------------------------------------------------------------*/

	void cut_bob (struct bob *bob,struct RastPort *rp,long x,long y)

	{
	struct RastPort temprp;
	if (bob)
		{
		InitRastPort (&temprp);
		temprp.BitMap = bob->bm;
		ClipBlit (rp,x,y,&temprp,0,0,bob->w,bob->h,0xc0);
		}
	}

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*--------------------------------------------------------------------------*/

	void remove_bob (struct bob *bob)

	{
	if (bob) if (bob->rp)
		{
		BltBitMap (bob->back,bob->backx,bob->backy,bob->rp->BitMap,bob->x + bob->backx,bob->y + bob->backy,bob->backw,bob->backh,0xc0,0xff,NULL);
		bob->rp = NULL;
		}
	}

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*--------------------------------------------------------------------------*/

	void add_bob (struct RastPort *rp,struct bob *bob)

	{
	long scrw,scrh,x,y;
	if (bob)
		{
		if (bob->rp)
			remove_bob (bob);
		bob->rp = rp;
		bob->backw = bob->w;
		bob->backh = bob->h;
		scrw = GetBitMapAttr (rp->BitMap,BMA_WIDTH);
		scrh = GetBitMapAttr (rp->BitMap,BMA_HEIGHT);
		if ((bob->x + bob->backw) > scrw)
			bob->backw = scrw - bob->x;
		if ((bob->y + bob->backh) > scrh)
			bob->backh = scrh - bob->y;
		bob->backx = 0;
		bob->backy = 0;
		if (bob->x < 0)
			{
			bob->backx = -bob->x;
			bob->backw -= bob->backx;
			}
		if (bob->y < 0)
			{
			bob->backy = -bob->y;
			bob->backh -= bob->backy;
			}
		x = bob->x + bob->backx;
		y = bob->y + bob->backy;
		BltBitMap (rp->BitMap,x,y,bob->back,bob->backx,bob->backy,bob->backw,bob->backh,0xc0,0xff,NULL);
		BltBitMap (bob->bm,bob->backx,bob->backy,rp->BitMap,x,y,bob->backw,bob->backh,0xc0,0xff,NULL);
		}
	}

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*--------------------------------------------------------------------------*/

	void move_bob (struct bob *bob,long x,long y)

	{
	struct RastPort *rp;
	if (bob)
		{
		if (rp = bob->rp)
			remove_bob (bob);
		bob->x = x;
		bob->y = y;
		if (rp)
			add_bob (rp,bob);
		}
	}

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*--------------------------------------------------------------------------*/

	void free_bob (struct bob *bob)

	{
	if (bob)
		{
		if (bob->rp)
			remove_bob (bob);
		if (bob->bm)
			FreeBitMap (bob->bm);
		if (bob->back)
			FreeBitMap (bob->back);
		FreeMem (bob,sizeof(struct bob));
		}
	}

/*--------------------------------------------------------------------------*/
/*                                                                          */
/*--------------------------------------------------------------------------*/

	struct bob *new_bob (long w,long h,long d)

	{
	struct bob *bob;
	if (bob = AllocMem (sizeof (struct bob),MEMF_CLEAR))
		{
		bob->w = w;
		bob->h = h;
		if (d > 8) d = 24;
		bob->bm = AllocBitMap (w,h,d,d == 24 ? BMF_SPECIALFMT|SHIFT_PIXFMT(PIXFMT_RGB24) : 0,NULL);
		bob->back = AllocBitMap (w,h,d,d == 24 ? BMF_SPECIALFMT|SHIFT_PIXFMT(PIXFMT_RGB24) : 0,NULL);
		if (!bob->bm || !bob->back)
			{
			free_bob (bob);
			bob = NULL;
			}
		}

	return (bob);
	}

/*--------------------------------------------------------------------------*/
/* Hauptprogramm                                                            */
/*--------------------------------------------------------------------------*/

	int main (int argc,char *argv[])

	{
	struct RDArgs *rdargs;
	static struct {
		char *pubscreen;
		} args = {NULL};
	struct Screen *scr;
	struct Window *win;
	struct MsgPort *port;
	struct IntuiMessage *mess;
	BOOL cont;
	struct RastPort *rp;
	long i;
	struct bob *bob;
	long diffx,diffy;

	if (rdargs = ReadArgs ("PUBSCREEN/K",(APTR)&args,NULL))
		{
		if (scr = LockPubScreen (args.pubscreen))
			{

			if (win = OpenWindowTags (NULL,
					WA_CustomScreen,scr,
					WA_Width,WINW,
					WA_Height,WINH,
					WA_Left,WINX,
					WA_Top,WINY,
					WA_Flags,WFLG_CLOSEGADGET|WFLG_DRAGBAR|WFLG_DEPTHGADGET|WFLG_REPORTMOUSE|WFLG_RMBTRAP|WFLG_GIMMEZEROZERO|WFLG_ACTIVATE,
					WA_IDCMP,IDCMP_CLOSEWINDOW|IDCMP_MOUSEMOVE|IDCMP_MOUSEBUTTONS,
					TAG_END))
				{
				rp = win->RPort;
				for (i = 0; i < BOBW; i++)
					{
					SetAPen (rp,(i & 3) + 1);
					Move (rp,i+BOBMINX,BOBMINY);
					Draw (rp,BOBMAXX-i,BOBMAXY);
					}
				for (i = 0; i < BOBH; i++)
					{
					SetAPen (rp,(i & 3) + 1);
					Move (rp,BOBMAXX,i+BOBMINY);
					Draw (rp,BOBMINX,BOBMAXY-i);
					}
				bob = new_bob (BOBW,BOBH,GetBitMapAttr (scr->RastPort.BitMap,BMA_DEPTH));
				cut_bob (bob,rp,BOBMINX,BOBMINY);
				RectFill (rp,BOBMINX,BOBMINY,BOBMAXX,BOBMAXY);
				rp = &scr->RastPort;
				port = win->UserPort;
				cont = TRUE;
				while (cont)
					{
					WaitPort (port);
					while (mess = (struct IntuiMessage *) GetMsg (port))
						{
						switch (mess->Class)
							{
						case IDCMP_CLOSEWINDOW:
							cont = FALSE;
							break;
						case IDCMP_MOUSEMOVE:
							move_bob (bob,scr->MouseX - diffx,scr->MouseY - diffy);
							break;
						case IDCMP_MOUSEBUTTONS:
							if (mess->Code == IECODE_LBUTTON)
								{
								if (win->GZZMouseX >= BOBMINX && win->GZZMouseX <= BOBMAXX && win->GZZMouseY >= BOBMINY && win->GZZMouseY <= BOBMAXY)
									{
									diffx = scr->MouseX - win->LeftEdge - win->BorderLeft - BOBMINX;
									diffy = scr->MouseY - win->TopEdge - win->BorderTop - BOBMINY;
									move_bob (bob,scr->MouseX - diffx,scr->MouseY - diffy);
									add_bob (&scr->RastPort,bob);
									}
								}
							else
								remove_bob (bob);
							break;
							}
						ReplyMsg ((struct Message *) mess);
						}
					}
				remove_bob (bob);
				free_bob (bob);
				CloseWindow (win);
				}

			UnlockPubScreen (NULL,scr);
			}

		FreeArgs (rdargs);
		}
	else
		PrintFault (IoErr(),NULL);

	return(0);
	}

/*--------------------------------------------------------------------------*/
/* Ende des Quelltextes                                                     */
/*--------------------------------------------------------------------------*/
